Package: broom.mixed
Type: Package
Title: Tidying Methods for Mixed Models
Version: 0.2.9.7
Authors@R: c(
    person("Ben", "Bolker", email = "bolker@mcmaster.ca",
        role = c("aut", "cre"),
        comment=c(ORCID="0000-0002-2127-0443")),
    person("David", "Robinson", email = "admiral.david@gmail.com",
        role = "aut"),
    person("Dieter", "Menne", role = "ctb"),
    person("Jonah", "Gabry", role = "ctb"),
    person("Paul", "Buerkner", role = "ctb"),
    person("Christopher", "Hua", role = "ctb"),
    person("William", "Petry", role = "ctb",
       comment=c(ORCID="0000-0002-5230-5987")),
    person("Joshua", "Wiley", role = "ctb",
       comment=c(ORCID="0000-0002-0271-6702")),
    person("Patrick", "Kennedy", role = "ctb"),
    person("Eduard", "Szöcs", role = "ctb",
       comment=c(ORCID = "0000-0001-5376-1194", sponsor = "BASF SE")),
    person("Indrajeet", "Patil", role="ctb"),
    person("Vincent", "Arel-Bundock",
          email = "vincent.arel-bundock@umontreal.ca",
          role = "ctb",
      comment = c(ORCID = "0000-0003-2042-7063")),
    person("Bill", "Denney", role = "ctb"),
    person("Cory", "Brunson", role = "ctb"),
    person("Joe", "Wasserman", role = "ctb",
        comment = c(ORCID = "0000-0002-9705-1853")),
    person("Alexey", "Stukalov", role = "ctb"),
    person("Matthieu", "Bruneaux", role = "ctb")
    )
Maintainer: Ben Bolker <bolker@mcmaster.ca>
Description: Convert fitted objects from various R mixed-model packages
    into tidy data frames along the lines of the 'broom' package.
    The package provides three
    S3 generics for each model: tidy(), which summarizes a model's statistical findings such as
    coefficients of a regression; augment(), which adds columns to the original
    data such as predictions, residuals and cluster assignments; and glance(), which
    provides a one-row summary of model-level statistics.
Imports: broom, coda (>= 0.19-4.1), dplyr, forcats, methods, nlme,
        purrr, stringr, tibble, tidyr, furrr
Suggests: brms, dotwhisker, knitr, testthat, gamlss, gamlss.data,
        ggplot2, GLMMadaptive, glmmADMB, glmmTMB, lmerTest, lme4,
        Matrix, MCMCglmm, mediation, mgcv, mice, ordinal, pander,
        pbkrtest, posterior, rstan, rstanarm, rstantools, R2jags, TMB,
        rmarkdown
URL: https://github.com/bbolker/broom.mixed
BugReports: https://github.com/bbolker/broom.mixed/issues
License: GPL-3
Encoding: UTF-8
Additional_repositories: http://bbolker.github.io/drat
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-02-16 21:59:45 UTC; bolker
Author: Ben Bolker [aut, cre] (ORCID: <https://orcid.org/0000-0002-2127-0443>),
  David Robinson [aut],
  Dieter Menne [ctb],
  Jonah Gabry [ctb],
  Paul Buerkner [ctb],
  Christopher Hua [ctb],
  William Petry [ctb] (ORCID: <https://orcid.org/0000-0002-5230-5987>),
  Joshua Wiley [ctb] (ORCID: <https://orcid.org/0000-0002-0271-6702>),
  Patrick Kennedy [ctb],
  Eduard Szöcs [ctb] (ORCID: <https://orcid.org/0000-0001-5376-1194>,
    sponsor: BASF SE),
  Indrajeet Patil [ctb],
  Vincent Arel-Bundock [ctb] (ORCID:
    <https://orcid.org/0000-0003-2042-7063>),
  Bill Denney [ctb],
  Cory Brunson [ctb],
  Joe Wasserman [ctb] (ORCID: <https://orcid.org/0000-0002-9705-1853>),
  Alexey Stukalov [ctb],
  Matthieu Bruneaux [ctb]
Repository: CRAN
Date/Publication: 2026-02-17 06:11:05 UTC
Built: R 4.6.0; ; 2026-04-25 13:33:48 UTC; unix
