# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,ranef.lme)
S3method(augment,brmsfit)
S3method(augment,clmm)
S3method(augment,gamm4)
S3method(augment,glmmTMB)
S3method(augment,glmmadmb)
S3method(augment,gls)
S3method(augment,lme)
S3method(augment,merMod)
S3method(augment,ranef.mer)
S3method(confint,rlmerMod)
S3method(fixef,MCMCglmm)
S3method(glance,TMB)
S3method(glance,allFit)
S3method(glance,brmsfit)
S3method(glance,gamlss)
S3method(glance,gamm4)
S3method(glance,glmmTMB)
S3method(glance,glmmadmb)
S3method(glance,gls)
S3method(glance,lme)
S3method(glance,lqmm)
S3method(glance,merMod)
S3method(glance,stanreg)
S3method(predict,clmm)
S3method(ranef,MCMCglmm)
S3method(sigma,brmsfit)
S3method(tidy,MCMCglmm)
S3method(tidy,MixMod)
S3method(tidy,TMB)
S3method(tidy,VarCorr.lme)
S3method(tidy,allFit)
S3method(tidy,brmsfit)
S3method(tidy,gamlss)
S3method(tidy,gamm4)
S3method(tidy,glmm)
S3method(tidy,glmmTMB)
S3method(tidy,glmmadmb)
S3method(tidy,gls)
S3method(tidy,lmList4)
S3method(tidy,lme)
S3method(tidy,lqmm)
S3method(tidy,mcmc)
S3method(tidy,mcmc.list)
S3method(tidy,mediate.mer)
S3method(tidy,merMod)
S3method(tidy,rjags)
S3method(tidy,rlmerMod)
S3method(tidy,stanfit)
S3method(tidy,stanreg)
S3method(tidy,varComb)
S3method(tidy,varFunc)
export(augment)
export(get_methods)
export(glance)
export(paramNamesMCMCglmm)
export(ranefLevels)
export(stdranef)
export(tidy)
export(tidyMCMC)
if(getRversion()>='3.3.0') importFrom(stats, sigma) else importFrom(lme4,sigma)
import(dplyr)
importFrom(broom,augment)
importFrom(broom,augment_columns)
importFrom(broom,glance)
importFrom(broom,tidy)
importFrom(dplyr,across)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,mutate)
importFrom(dplyr,tibble)
importFrom(forcats,fct_inorder)
importFrom(furrr,future_map_dfr)
importFrom(methods,is)
importFrom(methods,selectMethod)
importFrom(nlme,VarCorr)
importFrom(nlme,fixef)
importFrom(nlme,getVarCov)
importFrom(nlme,intervals)
importFrom(nlme,ranef)
importFrom(purrr,map)
importFrom(purrr,possibly)
importFrom(splines,backSpline)
importFrom(splines,interpSpline)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,cov2cor)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,na.omit)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,vcov)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,complete)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate)
importFrom(tidyr,spread)
importFrom(utils,globalVariables)
importFrom(utils,packageVersion)
