% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{unrowname}
\alias{unrowname}
\title{strip rownames from an object}
\usage{
unrowname(x)
}
\arguments{
\item{x}{a data frame}
}
\description{
strip rownames from an object
}
