% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{compact}
\alias{compact}
\title{Remove NULL items in a vector or list}
\usage{
compact(x)
}
\arguments{
\item{x}{a vector or list}
}
\description{
Remove NULL items in a vector or list
}
